@@ -3,10 +3,11 @@ PyQtGraph - Scientific Graphics and GUI Library for Python
 www.pyqtgraph.org
 """
 
-__version__ = '0.12.4'
+__version__ = '0.13.1'
 
 ### import all the goodies and add some helper functions for easy CLI use
 
+import importlib
 import os
 import sys
 
@@ -54,7 +55,11 @@ CONFIG_OPTIONS = {
                                  # change in the future.
     'useCupy': False,  # When True, attempt to use cupy ( currently only with ImageItem and related functions )
     'useNumba': False, # When True, use numba
-} 
+    'segmentedLineMode': 'auto',  # segmented line mode, controls if lines are plotted in segments or continuous
+                                  # 'auto': whether lines are plotted in segments is automatically decided using pen properties and whether anti-aliasing is enabled
+                                  # 'on' or True: lines are always plotted in segments
+                                  # 'off' or False: lines are never plotted in segments
+}
 
 
 def setConfigOption(opt, value):
@@ -62,6 +67,8 @@ def setConfigOption(opt, value):
         raise KeyError('Unknown configuration option "%s"' % opt)
     if opt == 'imageAxisOrder' and value not in ('row-major', 'col-major'):
         raise ValueError('imageAxisOrder must be either "row-major" or "col-major"')
+    if opt == 'segmentedLineMode' and value not in ('auto', 'on', 'off'):
+        raise ValueError('segmentedLineMode must be "auto", "on" or "off"')
     CONFIG_OPTIONS[opt] = value
 
 def setConfigOptions(**opts):
@@ -236,14 +243,11 @@ from .graphicsItems.VTickGroup import *
 
 # indirect imports used within library
 from .GraphicsScene import GraphicsScene
-from .graphicsWindows import *
 from .imageview import *
 
 # indirect imports known to be used outside of the library
 from .metaarray import MetaArray
-from .ordereddict import OrderedDict
 from .Point import Point
-from .ptime import time
 from .Qt import isQObjectAlive
 from .SignalProxy import *
 from .SRTTransform import SRTTransform
@@ -274,6 +278,7 @@ from .widgets.MultiPlotWidget import *
 from .widgets.PathButton import *
 from .widgets.PlotWidget import *
 from .widgets.ProgressDialog import *
+from .widgets.RawImageWidget import *
 from .widgets.RemoteGraphicsView import RemoteGraphicsView
 from .widgets.ScatterPlotWidget import *
 from .widgets.SpinBox import *
@@ -388,7 +393,7 @@ QAPP = None
 
 def plot(*args, **kargs):
     """
-    Create and return a :class:`PlotWidget <pyqtgraph.PlotWinPlotWidgetdow>` 
+    Create and return a :class:`PlotWidget <pyqtgraph.PlotWidget>` 
     Accepts a *title* argument to set the title of the window.
     All other arguments are used to plot data. (see :func:`PlotItem.plot() <pyqtgraph.PlotItem.plot>`)
     """
