@@ -1,17 +1,18 @@
-from ..Qt import QtGui, QtCore
-from ..python2_3 import sortList
-from .. import functions as fn
-from .GraphicsObject import GraphicsObject
-from .GraphicsWidget import GraphicsWidget
-from ..widgets.SpinBox import SpinBox
+import operator
 import weakref
-from ..pgcollections import OrderedDict
-from ..colormap import ColorMap
+from collections import OrderedDict
 
 import numpy as np
 
-__all__ = ['TickSliderItem', 'GradientEditorItem']
+from .. import functions as fn
+from ..colormap import ColorMap
+from ..Qt import QtCore, QtGui, QtWidgets
+from ..widgets.SpinBox import SpinBox
+from .GraphicsWidget import GraphicsWidget
 
+translate = QtCore.QCoreApplication.translate
+
+__all__ = ['TickSliderItem', 'GradientEditorItem']
 
 Gradients = OrderedDict([
     ('thermal', {'ticks': [(0.3333, (185, 0, 0, 255)), (0.6666, (255, 220, 0, 255)), (1, (255, 255, 255, 255)), (0, (0, 0, 0, 255))], 'mode': 'rgb'}),
@@ -22,9 +23,20 @@ Gradients = OrderedDict([
     ('cyclic', {'ticks': [(0.0, (255, 0, 4, 255)), (1.0, (255, 0, 0, 255))], 'mode': 'hsv'}),
     ('greyclip', {'ticks': [(0.0, (0, 0, 0, 255)), (0.99, (255, 255, 255, 255)), (1.0, (255, 0, 0, 255))], 'mode': 'rgb'}),
     ('grey', {'ticks': [(0.0, (0, 0, 0, 255)), (1.0, (255, 255, 255, 255))], 'mode': 'rgb'}),
+    # Perceptually uniform sequential colormaps from Matplotlib 2.0
+    ('viridis', {'ticks': [(0.0, (68, 1, 84, 255)), (0.25, (58, 82, 139, 255)), (0.5, (32, 144, 140, 255)), (0.75, (94, 201, 97, 255)), (1.0, (253, 231, 36, 255))], 'mode': 'rgb'}),
+    ('inferno', {'ticks': [(0.0, (0, 0, 3, 255)), (0.25, (87, 15, 109, 255)), (0.5, (187, 55, 84, 255)), (0.75, (249, 142, 8, 255)), (1.0, (252, 254, 164, 255))], 'mode': 'rgb'}),
+    ('plasma', {'ticks': [(0.0, (12, 7, 134, 255)), (0.25, (126, 3, 167, 255)), (0.5, (203, 71, 119, 255)), (0.75, (248, 149, 64, 255)), (1.0, (239, 248, 33, 255))], 'mode': 'rgb'}),
+    ('magma', {'ticks': [(0.0, (0, 0, 3, 255)), (0.25, (80, 18, 123, 255)), (0.5, (182, 54, 121, 255)), (0.75, (251, 136, 97, 255)), (1.0, (251, 252, 191, 255))], 'mode': 'rgb'}),
 ])
 
-
+def addGradientListToDocstring():
+    """Decorator to add list of current pre-defined gradients to the end of a function docstring."""
+    def dec(fn):
+        if fn.__doc__ is not None:
+            fn.__doc__ = fn.__doc__ + str(list(Gradients.keys())).strip('[').strip(']')
+        return fn
+    return dec
 
 
 
@@ -33,14 +45,18 @@ class TickSliderItem(GraphicsWidget):
     """**Bases:** :class:`GraphicsWidget <pyqtgraph.GraphicsWidget>`
     
     A rectangular item with tick marks along its length that can (optionally) be moved by the user."""
-        
-    def __init__(self, orientation='bottom', allowAdd=True, **kargs):
+    
+    sigTicksChanged = QtCore.Signal(object)
+    sigTicksChangeFinished = QtCore.Signal(object)
+    
+    def __init__(self, orientation='bottom', allowAdd=True, allowRemove=True, **kargs):
         """
         ==============  =================================================================================
         **Arguments:**
         orientation     Set the orientation of the gradient. Options are: 'left', 'right'
                         'top', and 'bottom'.
-        allowAdd        Specifies whether ticks can be added to the item by the user.
+        allowAdd        Specifies whether the user can add ticks.
+        allowRemove     Specifies whether the user can remove new ticks.
         tickPen         Default is white. Specifies the color of the outline of the ticks.
                         Can be any of the valid arguments for :func:`mkPen <pyqtgraph.mkPen>`
         ==============  =================================================================================
@@ -53,6 +69,7 @@ class TickSliderItem(GraphicsWidget):
         self.ticks = {}
         self.maxDim = 20
         self.allowAdd = allowAdd
+        self.allowRemove = allowRemove
         if 'tickPen' in kargs:
             self.tickPen = fn.mkPen(kargs['tickPen'])
         else:
@@ -66,8 +83,8 @@ class TickSliderItem(GraphicsWidget):
         }
         
         self.setOrientation(orientation)
-        #self.setFrameStyle(QtGui.QFrame.NoFrame | QtGui.QFrame.Plain)
-        #self.setBackgroundRole(QtGui.QPalette.NoRole)
+        #self.setFrameStyle(QtWidgets.QFrame.Shape.NoFrame | QtWidgets.QFrame.Shadow.Plain)
+        #self.setBackgroundRole(QtGui.QPalette.ColorRole.NoRole)
         #self.setMouseTracking(True)
         
     #def boundingRect(self):
@@ -117,22 +134,27 @@ class TickSliderItem(GraphicsWidget):
         self.resetTransform()
         ort = orientation
         if ort == 'top':
-            self.scale(1, -1)
-            self.translate(0, -self.height())
+            transform = QtGui.QTransform.fromScale(1, -1)
+            transform.translate(0, -self.height())
+            self.setTransform(transform)
         elif ort == 'left':
-            self.rotate(270)
-            self.scale(1, -1)
-            self.translate(-self.height(), -self.maxDim)
+            transform = QtGui.QTransform()
+            transform.rotate(270)
+            transform.scale(1, -1)
+            transform.translate(-self.height(), -self.maxDim)
+            self.setTransform(transform)
         elif ort == 'right':
-            self.rotate(270)
-            self.translate(-self.height(), 0)
-            #self.setPos(0, -self.height())
+            transform = QtGui.QTransform()
+            transform.rotate(270)
+            transform.translate(-self.height(), 0)
+            self.setTransform(transform)
         elif ort != 'bottom':
             raise Exception("%s is not a valid orientation. Options are 'left', 'right', 'top', and 'bottom'" %str(ort))
         
-        self.translate(self.tickSize/2., 0)
+        tr = QtGui.QTransform.fromTranslate(self.tickSize/2., 0)
+        self.setTransform(tr, True)
     
-    def addTick(self, x, color=None, movable=True):
+    def addTick(self, x, color=None, movable=True, finish=True):
         ## public
         """
         Add a tick to the item.
@@ -148,12 +170,22 @@ class TickSliderItem(GraphicsWidget):
         
         if color is None:
             color = QtGui.QColor(255,255,255)
-        tick = Tick(self, [x*self.length, 0], color, movable, self.tickSize, pen=self.tickPen)
+        tick = Tick([x*self.length, 0], color, movable, self.tickSize, pen=self.tickPen, removeAllowed=self.allowRemove)
         self.ticks[tick] = x
         tick.setParentItem(self)
+        
+        tick.sigMoving.connect(self.tickMoved)
+        tick.sigMoved.connect(self.tickMoveFinished)
+        tick.sigClicked.connect(self.tickClicked)
+        
+        self.sigTicksChanged.emit(self)
+        
+        if finish:
+            self.sigTicksChangeFinished.emit(self)
+        
         return tick
     
-    def removeTick(self, tick):
+    def removeTick(self, tick, finish=True):
         ## public
         """
         Removes the specified tick.
@@ -162,6 +194,11 @@ class TickSliderItem(GraphicsWidget):
         tick.setParentItem(None)
         if self.scene() is not None:
             self.scene().removeItem(tick)
+        
+        self.sigTicksChanged.emit(self)
+        
+        if finish:
+            self.sigTicksChangeFinished.emit(self)
     
     def tickMoved(self, tick, pos):
         #print "tick changed"
@@ -170,12 +207,14 @@ class TickSliderItem(GraphicsWidget):
         pos.setX(newX)
         tick.setPos(pos)
         self.ticks[tick] = float(newX) / self.length
+        
+        self.sigTicksChanged.emit(self)
     
     def tickMoveFinished(self, tick):
-        pass
+        self.sigTicksChangeFinished.emit(self)
     
     def tickClicked(self, tick, ev):
-        if ev.button() == QtCore.Qt.RightButton:
+        if ev.button() == QtCore.Qt.MouseButton.RightButton and tick.removeAllowed:
             self.removeTick(tick)
     
     def widgetLength(self):
@@ -192,7 +231,7 @@ class TickSliderItem(GraphicsWidget):
         #bounds.setLeft(min(-self.tickSize*0.5, bounds.left()))
         #bounds.setRight(max(self.length + self.tickSize, bounds.right()))
         #self.setSceneRect(bounds)
-        #self.fitInView(bounds, QtCore.Qt.KeepAspectRatio)
+        #self.fitInView(bounds, QtCore.Qt.AspectRatioMode.KeepAspectRatio)
         
     def setLength(self, newLen):
         #private
@@ -201,7 +240,7 @@ class TickSliderItem(GraphicsWidget):
         self.length = float(newLen)
         
     #def mousePressEvent(self, ev):
-        #QtGui.QGraphicsView.mousePressEvent(self, ev)
+        #QtWidgets.QGraphicsView.mousePressEvent(self, ev)
         #self.ignoreRelease = False
         #for i in self.items(ev.pos()):
             #if isinstance(i, Tick):
@@ -211,24 +250,24 @@ class TickSliderItem(GraphicsWidget):
             ##self.ignoreRelease = True
         
     #def mouseReleaseEvent(self, ev):
-        #QtGui.QGraphicsView.mouseReleaseEvent(self, ev)
+        #QtWidgets.QGraphicsView.mouseReleaseEvent(self, ev)
         #if self.ignoreRelease:
             #return
             
         #pos = self.mapToScene(ev.pos())
             
-        #if ev.button() == QtCore.Qt.LeftButton and self.allowAdd:
+        #if ev.button() == QtCore.Qt.MouseButton.LeftButton and self.allowAdd:
             #if pos.x() < 0 or pos.x() > self.length:
                 #return
             #if pos.y() < 0 or pos.y() > self.tickSize:
                 #return
             #pos.setX(min(max(pos.x(), 0), self.length))
             #self.addTick(pos.x()/self.length)
-        #elif ev.button() == QtCore.Qt.RightButton:
+        #elif ev.button() == QtCore.Qt.MouseButton.RightButton:
             #self.showMenu(ev)
             
     def mouseClickEvent(self, ev):
-        if ev.button() == QtCore.Qt.LeftButton and self.allowAdd:
+        if ev.button() == QtCore.Qt.MouseButton.LeftButton and self.allowAdd:
             pos = ev.pos()
             if pos.x() < 0 or pos.x() > self.length:
                 return
@@ -236,10 +275,10 @@ class TickSliderItem(GraphicsWidget):
                 return
             pos.setX(min(max(pos.x(), 0), self.length))
             self.addTick(pos.x()/self.length)
-        elif ev.button() == QtCore.Qt.RightButton:
+        elif ev.button() == QtCore.Qt.MouseButton.RightButton:
             self.showMenu(ev)
-        
-        #if  ev.button() == QtCore.Qt.RightButton:
+
+        #if  ev.button() == QtCore.Qt.MouseButton.RightButton:
             #if self.moving:
                 #ev.accept()
                 #self.setPos(self.startPosition)
@@ -252,8 +291,8 @@ class TickSliderItem(GraphicsWidget):
                 ###remove
 
     def hoverEvent(self, ev):
-        if (not ev.isExit()) and ev.acceptClicks(QtCore.Qt.LeftButton):
-            ev.acceptClicks(QtCore.Qt.RightButton)
+        if (not ev.isExit()) and ev.acceptClicks(QtCore.Qt.MouseButton.LeftButton):
+            ev.acceptClicks(QtCore.Qt.MouseButton.RightButton)
             ## show ghost tick
             #self.currentPen = fn.mkPen(255, 0,0)
         #else:
@@ -279,6 +318,9 @@ class TickSliderItem(GraphicsWidget):
         tick.color = color
         tick.update()
         #tick.setBrush(QtGui.QBrush(QtGui.QColor(tick.color)))
+        
+        self.sigTicksChanged.emit(self)
+        self.sigTicksChangeFinished.emit(self)
 
     def setTickValue(self, tick, val):
         ## public
@@ -301,7 +343,10 @@ class TickSliderItem(GraphicsWidget):
         pos.setX(x)
         tick.setPos(pos)
         self.ticks[tick] = val
-        self.updateGradient()
+        
+        self.update()
+        self.sigTicksChanged.emit(self)
+        self.sigTicksChangeFinished.emit(self)
         
     def tickValue(self, tick):
         ## public
@@ -332,13 +377,12 @@ class TickSliderItem(GraphicsWidget):
         return tick
 
     #def mouseMoveEvent(self, ev):
-        #QtGui.QGraphicsView.mouseMoveEvent(self, ev)
+        #QtWidgets.QGraphicsView.mouseMoveEvent(self, ev)
 
     def listTicks(self):
         """Return a sorted list of all the Tick objects on the slider."""
         ## public
-        ticks = list(self.ticks.items())
-        sortList(ticks, lambda a,b: cmp(a[1], b[1]))  ## see pyqtgraph.python2_3.sortList
+        ticks = sorted(self.ticks.items(), key=operator.itemgetter(1))
         return ticks
 
 
@@ -380,16 +424,16 @@ class GradientEditorItem(TickSliderItem):
         self.currentTick = None
         self.currentTickColor = None
         self.rectSize = 15
-        self.gradRect = QtGui.QGraphicsRectItem(QtCore.QRectF(0, self.rectSize, 100, self.rectSize))
-        self.backgroundRect = QtGui.QGraphicsRectItem(QtCore.QRectF(0, -self.rectSize, 100, self.rectSize))
-        self.backgroundRect.setBrush(QtGui.QBrush(QtCore.Qt.DiagCrossPattern))
+        self.gradRect = QtWidgets.QGraphicsRectItem(QtCore.QRectF(0, self.rectSize, 100, self.rectSize))
+        self.backgroundRect = QtWidgets.QGraphicsRectItem(QtCore.QRectF(0, -self.rectSize, 100, self.rectSize))
+        self.backgroundRect.setBrush(QtGui.QBrush(QtCore.Qt.BrushStyle.DiagCrossPattern))
         self.colorMode = 'rgb'
         
         TickSliderItem.__init__(self, *args, **kargs)
         
-        self.colorDialog = QtGui.QColorDialog()
-        self.colorDialog.setOption(QtGui.QColorDialog.ShowAlphaChannel, True)
-        self.colorDialog.setOption(QtGui.QColorDialog.DontUseNativeDialog, True)
+        self.colorDialog = QtWidgets.QColorDialog()
+        self.colorDialog.setOption(QtWidgets.QColorDialog.ColorDialogOption.ShowAlphaChannel, True)
+        self.colorDialog.setOption(QtWidgets.QColorDialog.ColorDialogOption.DontUseNativeDialog, True)
         
         self.colorDialog.currentColorChanged.connect(self.currentColorChanged)
         self.colorDialog.rejected.connect(self.currentColorRejected)
@@ -400,14 +444,14 @@ class GradientEditorItem(TickSliderItem):
         
         self.setMaxDim(self.rectSize + self.tickSize)
         
-        self.rgbAction = QtGui.QAction('RGB', self)
+        self.rgbAction = QtGui.QAction(translate("GradiantEditorItem", 'RGB'), self)
         self.rgbAction.setCheckable(True)
-        self.rgbAction.triggered.connect(lambda: self.setColorMode('rgb'))
-        self.hsvAction = QtGui.QAction('HSV', self)
+        self.rgbAction.triggered.connect(self._setColorModeToRGB)
+        self.hsvAction = QtGui.QAction(translate("GradiantEditorItem", 'HSV'), self)
         self.hsvAction.setCheckable(True)
-        self.hsvAction.triggered.connect(lambda: self.setColorMode('hsv'))
+        self.hsvAction.triggered.connect(self._setColorModeToHSV)
             
-        self.menu = QtGui.QMenu()
+        self.menu = QtWidgets.QMenu()
         
         ## build context menu of gradients
         l = self.length
@@ -421,11 +465,17 @@ class GradientEditorItem(TickSliderItem):
             brush = QtGui.QBrush(grad)
             p.fillRect(QtCore.QRect(0, 0, 100, 15), brush)
             p.end()
-            label = QtGui.QLabel()
+            label = QtWidgets.QLabel()
             label.setPixmap(px)
             label.setContentsMargins(1, 1, 1, 1)
-            act = QtGui.QWidgetAction(self)
-            act.setDefaultWidget(label)
+            labelName = QtWidgets.QLabel(g)
+            hbox = QtWidgets.QHBoxLayout()
+            hbox.addWidget(labelName)
+            hbox.addWidget(label)
+            widget = QtWidgets.QWidget()
+            widget.setLayout(hbox)
+            act = QtWidgets.QWidgetAction(self)
+            act.setDefaultWidget(widget)
             act.triggered.connect(self.contextMenuClicked)
             act.name = g
             self.menu.addAction(act)
@@ -441,7 +491,23 @@ class GradientEditorItem(TickSliderItem):
         self.addTick(1, QtGui.QColor(255,0,0), True)
         self.setColorMode('rgb')
         self.updateGradient()
-    
+        self.linkedGradients = {}
+        
+        self.sigTicksChanged.connect(self._updateGradientIgnoreArgs)
+        self.sigTicksChangeFinished.connect(self.sigGradientChangeFinished.emit)
+
+    def showTicks(self, show=True):
+        for tick in self.ticks.keys():
+            if show:
+                tick.show()
+                orig = getattr(self, '_allowAdd_backup', None)
+                if orig: 
+                    self.allowAdd = orig
+            else:
+                self._allowAdd_backup = self.allowAdd
+                self.allowAdd = False #block tick creation
+                tick.hide()
+
     def setOrientation(self, orientation):
         ## public
         """
@@ -456,7 +522,8 @@ class GradientEditorItem(TickSliderItem):
         ==============  ===================================================================
         """
         TickSliderItem.setOrientation(self, orientation)
-        self.translate(0, self.rectSize)
+        tr = QtGui.QTransform.fromTranslate(0, self.rectSize)
+        self.setTransform(tr, True)
     
     def showMenu(self, ev):
         #private
@@ -468,11 +535,12 @@ class GradientEditorItem(TickSliderItem):
         act = self.sender()
         self.loadPreset(act.name)
         
+    @addGradientListToDocstring()
     def loadPreset(self, name):
         """
-        Load a predefined gradient. 
-    
-        """ ## TODO: provide image with names of defined gradients
+        Load a predefined gradient. Currently defined gradients are: 
+        """## TODO: provide image with names of defined gradients
+        
         #global Gradients
         self.restoreState(Gradients[name])
     
@@ -495,8 +563,16 @@ class GradientEditorItem(TickSliderItem):
             self.rgbAction.blockSignals(False)
             self.hsvAction.blockSignals(False)
         self.colorMode = cm
-        self.updateGradient()
         
+        self.sigTicksChanged.emit(self)
+        self.sigGradientChangeFinished.emit(self)
+
+    def _setColorModeToRGB(self):
+        self.setColorMode("rgb")
+
+    def _setColorModeToHSV(self):
+        self.setColorMode("hsv")
+
     def colorMap(self):
         """Return a ColorMap object representing the current state of the editor."""
         if self.colorMode == 'hsv':
@@ -506,7 +582,7 @@ class GradientEditorItem(TickSliderItem):
         for t,x in self.listTicks():
             pos.append(x)
             c = t.color
-            color.append([c.red(), c.green(), c.blue(), c.alpha()])
+            color.append(c.getRgb())
         return ColorMap(np.array(pos), np.array(color, dtype=np.ubyte))
         
     def updateGradient(self):
@@ -514,33 +590,34 @@ class GradientEditorItem(TickSliderItem):
         self.gradient = self.getGradient()
         self.gradRect.setBrush(QtGui.QBrush(self.gradient))
         self.sigGradientChanged.emit(self)
-        
+
+    def _updateGradientIgnoreArgs(self, *args, **kwargs):
+        self.updateGradient()
+
     def setLength(self, newLen):
         #private (but maybe public)
         TickSliderItem.setLength(self, newLen)
         self.backgroundRect.setRect(1, -self.rectSize, newLen, self.rectSize)
         self.gradRect.setRect(1, -self.rectSize, newLen, self.rectSize)
-        self.updateGradient()
+        self.sigTicksChanged.emit(self)
         
     def currentColorChanged(self, color):
         #private
         if color.isValid() and self.currentTick is not None:
             self.setTickColor(self.currentTick, color)
-            self.updateGradient()
             
     def currentColorRejected(self):
         #private
         self.setTickColor(self.currentTick, self.currentTickColor)
-        self.updateGradient()
         
     def currentColorAccepted(self):
         self.sigGradientChangeFinished.emit(self)
         
     def tickClicked(self, tick, ev):
         #private
-        if ev.button() == QtCore.Qt.LeftButton:
+        if ev.button() == QtCore.Qt.MouseButton.LeftButton:
             self.raiseColorDialog(tick)
-        elif ev.button() == QtCore.Qt.RightButton:
+        elif ev.button() == QtCore.Qt.MouseButton.RightButton:
             self.raiseTickContextMenu(tick, ev)
             
     def raiseColorDialog(self, tick):
@@ -554,15 +631,9 @@ class GradientEditorItem(TickSliderItem):
     def raiseTickContextMenu(self, tick, ev):
         self.tickMenu = TickMenu(tick, self)
         self.tickMenu.popup(ev.screenPos().toQPoint())
-    
-    def tickMoved(self, tick, pos):
-        #private
-        TickSliderItem.tickMoved(self, tick, pos)
-        self.updateGradient()
 
     def tickMoveFinished(self, tick):
         self.sigGradientChangeFinished.emit(self)
-    
 
     def getGradient(self):
         """Return a QLinearGradient object."""
@@ -601,13 +672,13 @@ class GradientEditorItem(TickSliderItem):
             if toQColor:
                 return QtGui.QColor(c)  # always copy colors before handing them out
             else:
-                return (c.red(), c.green(), c.blue(), c.alpha())
+                return c.getRgb()
         if x >= ticks[-1][1]:
             c = ticks[-1][0].color
             if toQColor:
                 return QtGui.QColor(c)  # always copy colors before handing them out
             else:
-                return (c.red(), c.green(), c.blue(), c.alpha())
+                return c.getRgb()
             
         x2 = ticks[0][1]
         for i in range(1,len(ticks)):
@@ -638,12 +709,11 @@ class GradientEditorItem(TickSliderItem):
             h = h1 * (1.-f) + h2 * f
             s = s1 * (1.-f) + s2 * f
             v = v1 * (1.-f) + v2 * f
-            c = QtGui.QColor()
-            c.setHsv(h,s,v)
+            c = QtGui.QColor.fromHsv(int(h), int(s), int(v))
             if toQColor:
                 return c
             else:
-                return (c.red(), c.green(), c.blue(), c.alpha())
+                return c.getRgb()
                     
     def getLookupTable(self, nPts, alpha=None):
         """
@@ -687,17 +757,11 @@ class GradientEditorItem(TickSliderItem):
             return False
         if ticks[0][1] != 0.0 or ticks[1][1] != 1.0:
             return False
-        c1 = fn.colorTuple(ticks[0][0].color)
-        c2 = fn.colorTuple(ticks[1][0].color)
+        c1 = ticks[0][0].color.getRgb()
+        c2 = ticks[1][0].color.getRgb()
         if c1 != (0,0,0,255) or c2 != (255,255,255,255):
             return False
         return True
-
-
-    def mouseReleaseEvent(self, ev):
-        #private
-        TickSliderItem.mouseReleaseEvent(self, ev)
-        self.updateGradient()
         
     def addTick(self, x, color=None, movable=True, finish=True):
         """
@@ -712,24 +776,12 @@ class GradientEditorItem(TickSliderItem):
         ==============  ==================================================================
         """
         
-        
         if color is None:
             color = self.getColor(x)
-        t = TickSliderItem.addTick(self, x, color=color, movable=movable)
+        t = TickSliderItem.addTick(self, x, color=color, movable=movable, finish=finish)
         t.colorChangeAllowed = True
-        t.removeAllowed = True
         
-        if finish:
-            self.sigGradientChangeFinished.emit(self)
         return t
-
-
-    def removeTick(self, tick, finish=True):
-        TickSliderItem.removeTick(self, tick)
-        if finish:
-            self.updateGradient()
-            self.sigGradientChangeFinished.emit(self)
-        
         
     def saveState(self):
         """
@@ -742,8 +794,10 @@ class GradientEditorItem(TickSliderItem):
         ticks = []
         for t in self.ticks:
             c = t.color
-            ticks.append((self.ticks[t], (c.red(), c.green(), c.blue(), c.alpha())))
-        state = {'mode': self.colorMode, 'ticks': ticks}
+            ticks.append((self.ticks[t], c.getRgb()))
+        state = {'mode': self.colorMode, 
+                 'ticks': ticks,
+                 'ticksVisible': next(iter(self.ticks)).isVisible()}
         return state
         
     def restoreState(self, state):
@@ -762,16 +816,28 @@ class GradientEditorItem(TickSliderItem):
         ==============  ====================================================================
         """
         ## public
+        
+        # Mass edit ticks without graphics update
+        signalsBlocked = self.blockSignals(True)
+        
         self.setColorMode(state['mode'])
         for t in list(self.ticks.keys()):
             self.removeTick(t, finish=False)
         for t in state['ticks']:
             c = QtGui.QColor(*t[1])
             self.addTick(t[0], c, finish=False)
-        self.updateGradient()
+        self.showTicks( state.get('ticksVisible', 
+                                  next(iter(self.ticks)).isVisible()) )
+        
+        # Close with graphics update
+        self.blockSignals(signalsBlocked)
+        self.sigTicksChanged.emit(self)
         self.sigGradientChangeFinished.emit(self)
         
     def setColorMap(self, cm):
+        # Mass edit ticks without graphics update
+        signalsBlocked = self.blockSignals(True)
+        
         self.setColorMode('rgb')
         for t in list(self.ticks.keys()):
             self.removeTick(t, finish=False)
@@ -780,32 +846,52 @@ class GradientEditorItem(TickSliderItem):
             x = cm.pos[i]
             c = colors[i]
             self.addTick(x, c, finish=False)
-        self.updateGradient()
+        
+        # Close with graphics update
+        self.blockSignals(signalsBlocked)
+        self.sigTicksChanged.emit(self)
         self.sigGradientChangeFinished.emit(self)
 
-        
-class Tick(QtGui.QGraphicsObject):  ## NOTE: Making this a subclass of GraphicsObject instead results in 
+    def linkGradient(self, slaveGradient, connect=True):
+        if connect:
+            fn = lambda g, slave=slaveGradient:slave.restoreState(
+                                                     g.saveState())
+            self.linkedGradients[id(slaveGradient)] = fn
+            self.sigGradientChanged.connect(fn)
+            self.sigGradientChanged.emit(self)
+        else:
+            fn = self.linkedGradients.get(id(slaveGradient), None)
+            if fn:
+                self.sigGradientChanged.disconnect(fn)
+
+
+class Tick(QtWidgets.QGraphicsWidget):  ## NOTE: Making this a subclass of GraphicsObject instead results in
                                     ## activating this bug: https://bugreports.qt-project.org/browse/PYSIDE-86
     ## private class
-    
-    sigMoving = QtCore.Signal(object)
+
+    # When making Tick a subclass of QtWidgets.QGraphicsObject as origin,
+    # ..GraphicsScene.items(self, *args) will get Tick object as a
+    # class of QtGui.QMultimediaWidgets.QGraphicsVideoItem in python2.7-PyQt5(5.4.0)
+
+    sigMoving = QtCore.Signal(object, object)
     sigMoved = QtCore.Signal(object)
+    sigClicked = QtCore.Signal(object, object)
     
-    def __init__(self, view, pos, color, movable=True, scale=10, pen='w'):
+    def __init__(self, pos, color, movable=True, scale=10, pen='w', removeAllowed=True):
         self.movable = movable
         self.moving = False
-        self.view = weakref.ref(view)
         self.scale = scale
         self.color = color
         self.pen = fn.mkPen(pen)
         self.hoverPen = fn.mkPen(255,255,0)
         self.currentPen = self.pen
+        self.removeAllowed = removeAllowed
         self.pg = QtGui.QPainterPath(QtCore.QPointF(0,0))
         self.pg.lineTo(QtCore.QPointF(-scale/3**0.5, scale))
         self.pg.lineTo(QtCore.QPointF(scale/3**0.5, scale))
         self.pg.closeSubpath()
         
-        QtGui.QGraphicsObject.__init__(self)
+        QtWidgets.QGraphicsWidget.__init__(self)
         self.setPos(pos[0], pos[1])
         if self.movable:
             self.setZValue(1)
@@ -819,7 +905,7 @@ class Tick(QtGui.QGraphicsObject):  ## NOTE: Making this a subclass of GraphicsO
         return self.pg
 
     def paint(self, p, *args):
-        p.setRenderHints(QtGui.QPainter.Antialiasing)
+        p.setRenderHints(QtGui.QPainter.RenderHint.Antialiasing)
         p.fillPath(self.pg, fn.mkBrush(self.color))
         
         p.setPen(self.currentPen)
@@ -827,7 +913,7 @@ class Tick(QtGui.QGraphicsObject):  ## NOTE: Making this a subclass of GraphicsO
 
 
     def mouseDragEvent(self, ev):
-        if self.movable and ev.button() == QtCore.Qt.LeftButton:
+        if self.movable and ev.button() == QtCore.Qt.MouseButton.LeftButton:
             if ev.isStart():
                 self.moving = True
                 self.cursorOffset = self.pos() - self.mapToParent(ev.buttonDownPos())
@@ -841,50 +927,46 @@ class Tick(QtGui.QGraphicsObject):  ## NOTE: Making this a subclass of GraphicsO
             newPos.setY(self.pos().y())
             
             self.setPos(newPos)
-            self.view().tickMoved(self, newPos)
-            self.sigMoving.emit(self)
+            self.sigMoving.emit(self, newPos)
             if ev.isFinish():
                 self.moving = False
                 self.sigMoved.emit(self)
-                self.view().tickMoveFinished(self)
 
     def mouseClickEvent(self, ev):
-        if  ev.button() == QtCore.Qt.RightButton and self.moving:
-            ev.accept()
+        ev.accept()
+        if ev.button() == QtCore.Qt.MouseButton.RightButton and self.moving:
             self.setPos(self.startPosition)
-            self.view().tickMoved(self, self.startPosition)
             self.moving = False
-            self.sigMoving.emit(self)
+            self.sigMoving.emit(self, self.startPosition)
             self.sigMoved.emit(self)
         else:
-            self.view().tickClicked(self, ev)
-            ##remove
+            self.sigClicked.emit(self, ev)
 
     def hoverEvent(self, ev):
-        if (not ev.isExit()) and ev.acceptDrags(QtCore.Qt.LeftButton):
-            ev.acceptClicks(QtCore.Qt.LeftButton)
-            ev.acceptClicks(QtCore.Qt.RightButton)
+        if (not ev.isExit()) and ev.acceptDrags(QtCore.Qt.MouseButton.LeftButton):
+            ev.acceptClicks(QtCore.Qt.MouseButton.LeftButton)
+            ev.acceptClicks(QtCore.Qt.MouseButton.RightButton)
             self.currentPen = self.hoverPen
         else:
             self.currentPen = self.pen
         self.update()
         
 
-class TickMenu(QtGui.QMenu):
+class TickMenu(QtWidgets.QMenu):
     
     def __init__(self, tick, sliderItem):
-        QtGui.QMenu.__init__(self)
+        QtWidgets.QMenu.__init__(self)
         
         self.tick = weakref.ref(tick)
         self.sliderItem = weakref.ref(sliderItem)
         
-        self.removeAct = self.addAction("Remove Tick", lambda: self.sliderItem().removeTick(tick))
+        self.removeAct = self.addAction(translate("GradientEditorItem", "Remove Tick"), lambda: self.sliderItem().removeTick(tick))
         if (not self.tick().removeAllowed) or len(self.sliderItem().ticks) < 3:
             self.removeAct.setEnabled(False)
             
-        positionMenu = self.addMenu("Set Position")
-        w = QtGui.QWidget()
-        l = QtGui.QGridLayout()
+        positionMenu = self.addMenu(translate("GradientEditorItem", "Set Position"))
+        w = QtWidgets.QWidget()
+        l = QtWidgets.QGridLayout()
         w.setLayout(l)
         
         value = sliderItem.tickValue(tick)
@@ -893,22 +975,22 @@ class TickMenu(QtGui.QMenu):
         #self.dataPosSpin = SpinBox(value=dataVal)
         #self.dataPosSpin.setOpts(decimals=3, siPrefix=True)
                 
-        l.addWidget(QtGui.QLabel("Position:"), 0,0)
+        l.addWidget(QtWidgets.QLabel(f"{translate('GradiantEditorItem', 'Position')}:"), 0,0)
         l.addWidget(self.fracPosSpin, 0, 1)
-        #l.addWidget(QtGui.QLabel("Position (data units):"), 1, 0)
+        #l.addWidget(QtWidgets.QLabel("Position (data units):"), 1, 0)
         #l.addWidget(self.dataPosSpin, 1,1)
         
         #if self.sliderItem().dataParent is None:
         #    self.dataPosSpin.setEnabled(False)
         
-        a = QtGui.QWidgetAction(self)
+        a = QtWidgets.QWidgetAction(self)
         a.setDefaultWidget(w)
         positionMenu.addAction(a)        
         
         self.fracPosSpin.sigValueChanging.connect(self.fractionalValueChanged)
         #self.dataPosSpin.valueChanged.connect(self.dataValueChanged)
         
-        colorAct = self.addAction("Set Color", lambda: self.sliderItem().raiseColorDialog(self.tick()))
+        colorAct = self.addAction(translate("Context Menu", "Set Color"), lambda: self.sliderItem().raiseColorDialog(self.tick()))
         if not self.tick().colorChangeAllowed:
             colorAct.setEnabled(False)
 
@@ -924,4 +1006,3 @@ class TickMenu(QtGui.QMenu):
     #    self.fracPosSpin.blockSignals(True)
     #    self.fracPosSpin.setValue(self.sliderItem().tickValue(self.tick()))
     #    self.fracPosSpin.blockSignals(False)
-
