@@ -64,11 +64,11 @@ class HistogramLUTItem(GraphicsWidget):
 
     Attributes
     ----------
-    sigLookupTableChanged : signal
+    sigLookupTableChanged : QtCore.Signal
         Emits the HistogramLUTItem itself when the gradient changes
-    sigLevelsChanged : signal
+    sigLevelsChanged : QtCore.Signal
         Emits the HistogramLUTItem itself while the movable region is changing
-    sigLevelChangeFinished : signal
+    sigLevelChangeFinished : QtCore.Signal
         Emits the HistogramLUTItem itself when the movable region is finished changing
 
     See Also
@@ -195,7 +195,7 @@ class HistogramLUTItem(GraphicsWidget):
         level : float, optional
             Set the fill level. See :meth:`PlotCurveItem.setFillLevel
             <pyqtgraph.PlotCurveItem.setFillLevel>`. Only used if ``fill`` is True.
-        color : color, optional
+        color : color_like, optional
             Color to use for the fill when the histogram ``levelMode == "mono"``. See
             :meth:`PlotCurveItem.setBrush <pyqtgraph.PlotCurveItem.setBrush>`.
         """
@@ -254,7 +254,7 @@ class HistogramLUTItem(GraphicsWidget):
                 p.drawLine(gradRect.topRight(), gradRect.bottomRight())
 
     def setHistogramRange(self, mn, mx, padding=0.1):
-        """Set the Y range on the histogram plot. This disables auto-scaling."""
+        """Set the X/Y range on the histogram plot, depending on the orientation. This disables auto-scaling."""
         if self.orientation == 'vertical':
             self.vb.enableAutoRange(self.vb.YAxis, False)
             self.vb.setYRange(mn, mx, padding)
@@ -262,6 +262,13 @@ class HistogramLUTItem(GraphicsWidget):
             self.vb.enableAutoRange(self.vb.XAxis, False)
             self.vb.setXRange(mn, mx, padding)
 
+    def getHistogramRange(self):
+        """Returns range on the histogram plot."""
+        if self.orientation == 'vertical':
+            return self.vb.viewRange()[1]
+        else:
+            return self.vb.viewRange()[0]
+
     def autoHistogramRange(self):
         """Enable auto-scaling on the histogram plot."""
         self.vb.enableAutoRange(self.vb.XYAxes)
@@ -430,7 +437,8 @@ class HistogramLUTItem(GraphicsWidget):
 
         # force this because calling self.setLevels might not set the imageItem
         # levels if there was no change to the region item
-        self.imageItem().setLevels(self.getLevels())
+        if self.imageItem() is not None:
+            self.imageItem().setLevels(self.getLevels())
 
         self.imageChanged()
         self.update()
