@@ -16,29 +16,59 @@ from .GraphicsObject import GraphicsObject
 __all__ = ['PlotCurveItem']
 
 
-if Qt.QT_LIB.startswith('PyQt'):
-    wrapinstance = Qt.sip.wrapinstance
-else:
-    wrapinstance = Qt.shiboken.wrapInstance
+def have_native_drawlines_array():
+    size = 10
+    line = QtCore.QLineF(0, 0, size, size)
+    qimg = QtGui.QImage(size, size, QtGui.QImage.Format.Format_RGB32)
+    qimg.fill(QtCore.Qt.GlobalColor.transparent)
+    painter = QtGui.QPainter(qimg)
+    painter.setPen(QtCore.Qt.GlobalColor.white)
+
+    try:
+        painter.drawLines(line, 1)
+    except TypeError:
+        success = False
+    else:
+        success = True
+    finally:
+        painter.end()
+
+    return success
+
+_have_native_drawlines_array = Qt.QT_LIB.startswith('PySide') and have_native_drawlines_array()
 
 
 class LineSegments:
     def __init__(self):
+        self.use_sip_array = (
+            Qt.QT_LIB.startswith('PyQt') and
+            hasattr(Qt.sip, 'array') and
+            (
+                (0x60301 <= QtCore.PYQT_VERSION) or
+                (0x50f07 <= QtCore.PYQT_VERSION < 0x60000)
+            )
+        )
+        self.use_native_drawlines = Qt.QT_LIB.startswith('PySide') and _have_native_drawlines_array
         self.alloc(0)
 
     def alloc(self, size):
-        self.arr = np.empty((size, 4), dtype=np.float64)
-        self.ptrs = list(map(wrapinstance,
-            itertools.count(self.arr.ctypes.data, self.arr.strides[0]),
-            itertools.repeat(QtCore.QLineF, self.arr.shape[0])))
+        if self.use_sip_array:
+            self.objs = Qt.sip.array(QtCore.QLineF, size)
+            vp = Qt.sip.voidptr(self.objs, len(self.objs)*4*8)
+            self.arr = np.frombuffer(vp, dtype=np.float64).reshape((-1, 4))
+        elif self.use_native_drawlines:
+            self.arr = np.empty((size, 4), dtype=np.float64)
+            self.objs = Qt.compat.wrapinstance(self.arr.ctypes.data, QtCore.QLineF)
+        else:
+            self.arr = np.empty((size, 4), dtype=np.float64)
+            self.objs = list(map(Qt.compat.wrapinstance,
+                itertools.count(self.arr.ctypes.data, self.arr.strides[0]),
+                itertools.repeat(QtCore.QLineF, self.arr.shape[0])))
 
-    def array(self, size):
-        if size > self.arr.shape[0]:
-            self.alloc(size + 16)
-        return self.arr[:size]
-
-    def instances(self, size):
-        return self.ptrs[:size]
+    def get(self, size):
+        if size != self.arr.shape[0]:
+            self.alloc(size)
+        return self.objs, self.arr
 
     def arrayToLineSegments(self, x, y, connect, finiteCheck):
         # analogue of arrayToQPath taking the same parameters
@@ -47,7 +77,8 @@ class LineSegments:
 
         connect_array = None
         if isinstance(connect, np.ndarray):
-            connect_array, connect = connect, 'array'
+            # the last element is not used
+            connect_array, connect = np.asarray(connect[:-1], dtype=bool), 'array'
 
         all_finite = True
         if finiteCheck or connect == 'finite':
@@ -57,11 +88,13 @@ class LineSegments:
         if connect == 'all':
             if not all_finite:
                 # remove non-finite points, if any
-                x = x.compress(mask)
-                y = y.compress(mask)
+                x = x[mask]
+                y = y[mask]
 
         elif connect == 'finite':
-            if not all_finite:
+            if all_finite:
+                connect = 'all'
+            else:
                 # each non-finite point affects the segment before and after
                 connect_array = mask[:-1] & mask[1:]
 
@@ -72,30 +105,42 @@ class LineSegments:
                 x = x[backfill_idx]
                 y = y[backfill_idx]
 
-        npts = len(x)
-        if npts < 2:
-            return []
-
         segs = []
+        nsegs = 0
 
-        if connect in ['all', 'finite', 'array']:
-            memory = self.array(npts - 1)
-            memory[:, 0] = x[:-1]
-            memory[:, 1] = y[:-1]
-            memory[:, 2] = x[1:]
-            memory[:, 3] = y[1:]
-            segs = self.instances(npts - 1)
-            if connect_array is not None:
-                segs = list(itertools.compress(segs, connect_array.tolist()))
+        if connect == 'all':
+            nsegs = len(x) - 1
+            if nsegs:
+                segs, memory = self.get(nsegs)
+                memory[:, 0] = x[:-1]
+                memory[:, 2] = x[1:]
+                memory[:, 1] = y[:-1]
+                memory[:, 3] = y[1:]
 
-        elif connect in ['pairs']:
-            npairs = npts // 2
-            memory = self.array(npairs).reshape((-1, 2))
-            memory[:, 0] = x[:npairs * 2]
-            memory[:, 1] = y[:npairs * 2]
-            segs = self.instances(npairs)
+        elif connect == 'pairs':
+            nsegs = len(x) // 2
+            if nsegs:
+                segs, memory = self.get(nsegs)
+                memory = memory.reshape((-1, 2))
+                memory[:, 0] = x[:nsegs * 2]
+                memory[:, 1] = y[:nsegs * 2]
 
-        return segs
+        elif connect_array is not None:
+            # the following are handled here
+            # - 'array'
+            # - 'finite' with non-finite elements
+            nsegs = np.count_nonzero(connect_array)
+            if nsegs:
+                segs, memory = self.get(nsegs)
+                memory[:, 0] = x[:-1][connect_array]
+                memory[:, 2] = x[1:][connect_array]
+                memory[:, 1] = y[:-1][connect_array]
+                memory[:, 3] = y[1:][connect_array]
+
+        if nsegs and self.use_native_drawlines:
+            return segs, nsegs
+        else:
+            return segs,
 
 
 class PlotCurveItem(GraphicsObject):
@@ -151,7 +196,8 @@ class PlotCurveItem(GraphicsObject):
             'connect': 'all',
             'mouseWidth': 8, # width of shape responding to mouse click
             'compositionMode': None,
-            'skipFiniteCheck': False
+            'skipFiniteCheck': False,
+            'segmentedLineMode': getConfigOption('segmentedLineMode'),
         }
         if 'pen' not in kargs:
             self.opts['pen'] = fn.mkPen('w')
@@ -274,7 +320,7 @@ class PlotCurveItem(GraphicsObject):
         ## Add pen width only if it is non-cosmetic.
         pen = self.opts['pen']
         spen = self.opts['shadowPen']
-        if not pen.isCosmetic():
+        if pen is not None and not pen.isCosmetic() and pen.style() != QtCore.Qt.PenStyle.NoPen:
             b = (b[0] - pen.widthF()*0.7072, b[1] + pen.widthF()*0.7072)
         if spen is not None and not spen.isCosmetic() and spen.style() != QtCore.Qt.PenStyle.NoPen:
             b = (b[0] - spen.widthF()*0.7072, b[1] + spen.widthF()*0.7072)
@@ -286,7 +332,7 @@ class PlotCurveItem(GraphicsObject):
         pen = self.opts['pen']
         spen = self.opts['shadowPen']
         w = 0
-        if pen.isCosmetic():
+        if  pen is not None and pen.isCosmetic() and pen.style() != QtCore.Qt.PenStyle.NoPen:
             w += pen.widthF()*0.7072
         if spen is not None and spen.isCosmetic() and spen.style() != QtCore.Qt.PenStyle.NoPen:
             w = max(w, spen.widthF()*0.7072)
@@ -624,7 +670,35 @@ class PlotCurveItem(GraphicsObject):
 
         return self.path
 
+    def setSegmentedLineMode(self, mode):
+        """
+        Sets the mode that decides whether or not lines are drawn as segmented lines. Drawing lines
+        as segmented lines is more performant than the standard drawing method with continuous
+        lines.
+
+        Parameters
+        ----------
+        mode : str
+               ``'auto'`` (default) segmented lines are drawn if the pen's width > 1, pen style is a
+               solid line, the pen color is opaque and anti-aliasing is not enabled.
+
+               ``'on'`` lines are always drawn as segmented lines
+
+               ``'off'`` lines are never drawn as segmented lines, i.e. the drawing
+               method with continuous lines is used
+        """
+        if mode not in ('auto', 'on', 'off'):
+            raise ValueError(f'segmentedLineMode must be "auto", "on" or "off", got {mode} instead')
+        self.opts['segmentedLineMode'] = mode
+        self.invalidateBounds()
+        self.update()
+
     def _shouldUseDrawLineSegments(self, pen):
+        mode = self.opts['segmentedLineMode']
+        if mode in ('on',):
+            return True
+        if mode in ('off',):
+            return False
         return (
             pen.widthF() > 1.0
             # non-solid pen styles need single polyline to be effective
@@ -634,6 +708,9 @@ class PlotCurveItem(GraphicsObject):
             and pen.isSolid()   # pen.brush().style() == Qt.BrushStyle.SolidPattern
             # ends of adjacent line segments overlapping is visible when not opaque
             and pen.color().alphaF() == 1.0
+            # anti-aliasing introduces transparent pixels and therefore also causes visible overlaps
+            # for adjacent line segments
+            and not self.opts['antialias']
         )
 
     def _getLineSegments(self):
@@ -710,7 +787,7 @@ class PlotCurveItem(GraphicsObject):
             and isinstance(self.opts['fillLevel'], (int, float))
         )
 
-    def _getFillPathList(self):
+    def _getFillPathList(self, widget):
         if self._fillPathList is not None:
             return self._fillPathList
 
@@ -729,15 +806,22 @@ class PlotCurveItem(GraphicsObject):
             if not mask.all():
                 # we are only supporting connect='all',
                 # so remove non-finite values
-                x = x.compress(mask)
-                y = y.compress(mask)
+                x = x[mask]
+                y = y[mask]
 
         if len(x) < 2:
             return []
 
+        # Set suitable chunk size for current configuration:
+        #   * Without OpenGL split in small chunks
+        #   * With OpenGL split in rather big chunks
+        #     Note, the present code is used only if config option 'enableExperimental' is False,
+        #     otherwise the 'paintGL' method is used.
+        # Values were found using 'PlotSpeedTest.py' example, see #2257.
+        chunksize = 50 if not isinstance(widget, QtWidgets.QOpenGLWidget) else 5000
+
         paths = self._fillPathList = []
         offset = 0
-        chunksize = 50          # determined empirically
         xybuf = np.empty((chunksize+3, 2))
         baseline = self.opts['fillLevel']
 
@@ -784,7 +868,7 @@ class PlotCurveItem(GraphicsObject):
 
         if do_fill:
             if self._shouldUseFillPathList():
-                paths = self._getFillPathList()
+                paths = self._getFillPathList(widget)
             else:
                 paths = [self._getFillPath()]
 
@@ -803,7 +887,7 @@ class PlotCurveItem(GraphicsObject):
             if sp.style() != QtCore.Qt.PenStyle.NoPen:
                 p.setPen(sp)
                 if self._shouldUseDrawLineSegments(sp):
-                    p.drawLines(self._getLineSegments())
+                    p.drawLines(*self._getLineSegments())
                     if do_fill_outline:
                         p.drawLines(self._getClosingSegments())
                 else:
@@ -818,7 +902,7 @@ class PlotCurveItem(GraphicsObject):
 
         p.setPen(cp)
         if self._shouldUseDrawLineSegments(cp):
-            p.drawLines(self._getLineSegments())
+            p.drawLines(*self._getLineSegments())
             if do_fill_outline:
                 p.drawLines(self._getClosingSegments())
         else:
