@@ -1,6 +1,5 @@
 import itertools
 import math
-import warnings
 import weakref
 from collections import OrderedDict
 
@@ -97,7 +96,8 @@ def renderSymbol(symbol, size, pen, brush, device=None):
     ## Render a spot with the given parameters to a pixmap
     penPxWidth = max(math.ceil(pen.widthF()), 1)
     if device is None:
-        device = QtGui.QImage(int(size+penPxWidth), int(size+penPxWidth), QtGui.QImage.Format.Format_ARGB32)
+        device = QtGui.QImage(int(size+penPxWidth), int(size+penPxWidth),
+            QtGui.QImage.Format.Format_ARGB32_Premultiplied)
         device.fill(QtCore.Qt.GlobalColor.transparent)
     p = QtGui.QPainter(device)
     try:
@@ -108,15 +108,6 @@ def renderSymbol(symbol, size, pen, brush, device=None):
         p.end()
     return device
 
-def makeSymbolPixmap(size, pen, brush, symbol):
-    warnings.warn(
-        "This is an internal function that is no longer being used. "
-        "Will be removed in 0.13",
-        DeprecationWarning, stacklevel=2
-    )
-    img = renderSymbol(symbol, size, pen, brush)
-    return QtGui.QPixmap(img)
-
 
 def _mkPen(*args, **kwargs):
     """
@@ -144,6 +135,14 @@ def _mkBrush(*args, **kwargs):
 
 class PixmapFragments:
     def __init__(self):
+        self.use_sip_array = (
+            Qt.QT_LIB.startswith('PyQt') and
+            hasattr(Qt.sip, 'array') and
+            (
+                (0x60301 <= QtCore.PYQT_VERSION) or
+                (0x50f07 <= QtCore.PYQT_VERSION < 0x60000)
+            )
+        )
         self.alloc(0)
 
     def alloc(self, size):
@@ -158,24 +157,31 @@ class PixmapFragments:
         #    - this is mitigated here by reusing the instance pointers
         # 2) PyQt will anyway deconstruct the Python list and repack the PixmapFragment
         #    instances into a contiguous array, in order to call the underlying C++ native API.
-        self.arr = np.empty((size, 10), dtype=np.float64)
-        if QT_LIB.startswith('PyQt'):
-            self.ptrs = list(map(Qt.sip.wrapinstance,
-                itertools.count(self.arr.ctypes.data, self.arr.strides[0]),
-                itertools.repeat(QtGui.QPainter.PixmapFragment, self.arr.shape[0])))
+        if self.use_sip_array:
+            self.objs = Qt.sip.array(QtGui.QPainter.PixmapFragment, size)
+            vp = Qt.sip.voidptr(self.objs, len(self.objs)*10*8)
+            self.arr = np.frombuffer(vp, dtype=np.float64).reshape((-1, 10))
         else:
-            self.ptrs = Qt.shiboken.wrapInstance(self.arr.ctypes.data, QtGui.QPainter.PixmapFragment)
+            self.arr = np.empty((size, 10), dtype=np.float64)
+            if QT_LIB.startswith('PyQt'):
+                self.objs = list(map(Qt.sip.wrapinstance,
+                    itertools.count(self.arr.ctypes.data, self.arr.strides[0]),
+                    itertools.repeat(QtGui.QPainter.PixmapFragment, self.arr.shape[0])))
+            else:
+                self.objs = Qt.shiboken.wrapInstance(self.arr.ctypes.data, QtGui.QPainter.PixmapFragment)
 
     def array(self, size):
-        if size > self.arr.shape[0]:
-            self.alloc(size + 16)
-        return self.arr[:size]
+        if size != self.arr.shape[0]:
+            self.alloc(size)
+        return self.arr
 
-    def draw(self, painter, size, pixmap):
+    def draw(self, painter, pixmap):
+        if not len(self.arr):
+            return
         if QT_LIB.startswith('PyQt'):
-            painter.drawPixmapFragments(self.ptrs[:size], pixmap)
+            painter.drawPixmapFragments(self.objs, pixmap)
         else:
-            painter.drawPixmapFragments(self.ptrs, size, pixmap)
+            painter.drawPixmapFragments(self.objs, len(self.arr), pixmap)
 
 
 class SymbolAtlas(object):
@@ -229,7 +235,7 @@ class SymbolAtlas(object):
         return self._maxWidth
 
     def rebuild(self, styles=None):
-        profiler = debug.Profiler()
+        profiler = debug.Profiler()  # noqa: profiler prints on GC
         if styles is None:
             data = []
         else:
@@ -279,7 +285,7 @@ class SymbolAtlas(object):
         data = []
         for key, style in styles.items():
             img = renderSymbol(*style)
-            arr = fn.imageToArray(img, copy=False, transpose=False)
+            arr = fn.ndarray_from_qimage(img)
             images.append(img)  # keep these to delay garbage collection
             data.append((key, arr))
 
@@ -353,11 +359,12 @@ class SymbolAtlas(object):
         return int(w), int(y + h)
 
     def _createPixmap(self):
-        profiler = debug.Profiler()
+        profiler = debug.Profiler()  # noqa: profiler prints on GC
         if self._data.size == 0:
             pm = QtGui.QPixmap(0, 0)
         else:
-            img = fn.makeQImage(self._data, copy=False, transpose=False)
+            img = fn.ndarray_to_qimage(self._data,
+                QtGui.QImage.Format.Format_ARGB32_Premultiplied)
             pm = QtGui.QPixmap(img)
         return pm
 
@@ -443,6 +450,10 @@ class ScatterPlotItem(GraphicsObject):
 
         #self.setCacheMode(self.DeviceCoordinateCache)
 
+        # track when the tooltip is cleared so we only clear it once
+        # this allows another item in the VB to set the tooltip
+        self._toolTipCleared = True
+
     def setData(self, *args, **kargs):
         """
         **Ordered Arguments:**
@@ -479,6 +490,9 @@ class ScatterPlotItem(GraphicsObject):
         *hoverSize*            A single size to use for hovered spots. Set to -1 to keep size unchanged. Default is -1.
         *hoverPen*             A single pen to use for hovered spots. Set to None to keep pen unchanged. Default is None.
         *hoverBrush*           A single brush to use for hovered spots. Set to None to keep brush unchanged. Default is None.
+        *useCache*             (bool) By default, generated point graphics items are cached to
+                               improve performance. Setting this to False can improve image quality
+                               in certain situations.
         *antialias*            Whether to draw symbols with antialiasing. Note that if pxMode is True, symbols are
                                always rendered with antialiasing (since the rendered symbols can be cached, this
                                incurs very little performance cost)
@@ -488,12 +502,6 @@ class ScatterPlotItem(GraphicsObject):
                                generating LegendItem entries and by some exporters.
         ====================== ===============================================================================================
         """
-        if 'identical' in kargs:
-            warnings.warn(
-                "The *identical* functionality is handled automatically now. "
-                "Will be removed in 0.13.",
-                DeprecationWarning, stacklevel=2
-            )
         oldData = self.data  ## this causes cached pixmaps to be preserved while new data is registered.
         self.clear()  ## clear out all old data
         self.addPoints(*args, **kargs)
@@ -593,6 +601,8 @@ class ScatterPlotItem(GraphicsObject):
             self.opts['hoverable'] = bool(kargs['hoverable'])
         if 'tip' in kargs:
             self.opts['tip'] = kargs['tip']
+        if 'useCache' in kargs:
+            self.opts['useCache'] = kargs['useCache']
 
         ## Set any extra parameters provided in keyword arguments
         for k in ['pen', 'brush', 'symbol', 'size']:
@@ -625,14 +635,6 @@ class ScatterPlotItem(GraphicsObject):
     def getData(self):
         return self.data['x'], self.data['y']
 
-    def setPoints(self, *args, **kargs):
-        warnings.warn(
-            "ScatterPlotItem.setPoints is deprecated, use ScatterPlotItem.setData "
-            "instead.  Will be removed in 0.13",
-            DeprecationWarning, stacklevel=2
-        )
-        return self.setData(*args, **kargs)
-
     def implements(self, interface=None):
         ints = ['plotData']
         if interface is None:
@@ -803,8 +805,7 @@ class ScatterPlotItem(GraphicsObject):
         self.invalidate()
 
     def updateSpots(self, dataSet=None):
-        profiler = debug.Profiler()
-
+        profiler = debug.Profiler()  # noqa: profiler prints on GC
         if dataSet is None:
             dataSet = self.data
 
@@ -884,58 +885,6 @@ class ScatterPlotItem(GraphicsObject):
                 else:
                     yield size + pen.widthF(), 0
 
-    def getSpotOpts(self, recs, scale=1.0):
-        warnings.warn(
-            "This is an internal method that is no longer being used.  Will be "
-            "removed in 0.13",
-            DeprecationWarning, stacklevel=2
-        )
-        if recs.ndim == 0:
-            rec = recs
-            symbol = rec['symbol']
-            if symbol is None:
-                symbol = self.opts['symbol']
-            size = rec['size']
-            if size < 0:
-                size = self.opts['size']
-            pen = rec['pen']
-            if pen is None:
-                pen = self.opts['pen']
-            brush = rec['brush']
-            if brush is None:
-                brush = self.opts['brush']
-            return (symbol, size*scale, fn.mkPen(pen), fn.mkBrush(brush))
-        else:
-            recs = recs.copy()
-            recs['symbol'][np.equal(recs['symbol'], None)] = self.opts['symbol']
-            recs['size'][np.equal(recs['size'], -1)] = self.opts['size']
-            recs['size'] *= scale
-            recs['pen'][np.equal(recs['pen'], None)] = fn.mkPen(self.opts['pen'])
-            recs['brush'][np.equal(recs['brush'], None)] = fn.mkBrush(self.opts['brush'])
-            return recs
-
-    def measureSpotSizes(self, dataSet):
-        warnings.warn(
-            "This is an internal method that is no longer being used. "
-            "Will be removed in 0.13.",
-            DeprecationWarning, stacklevel=2
-        )
-        for size, pen in zip(*self._style(['size', 'pen'], data=dataSet)):
-            ## keep track of the maximum spot size and pixel size
-            width = 0
-            pxWidth = 0
-            if self.opts['pxMode']:
-                pxWidth = size + pen.widthF()
-            else:
-                width = size
-                if pen.isCosmetic():
-                    pxWidth += pen.widthF()
-                else:
-                    width += pen.widthF()
-            self._maxSpotWidth = max(self._maxSpotWidth, width)
-            self._maxSpotPxWidth = max(self._maxSpotPxWidth, pxWidth)
-        self.bounds = [None, None]
-
     def clear(self):
         """Remove all spots from the scatter plot"""
         #self.clearItems()
@@ -965,7 +914,6 @@ class ScatterPlotItem(GraphicsObject):
         if orthoRange is not None:
             mask = (d2 >= orthoRange[0]) * (d2 <= orthoRange[1])
             d = d[mask]
-            d2 = d2[mask]
 
             if d.size == 0:
                 return (None, None)
@@ -1021,44 +969,6 @@ class ScatterPlotItem(GraphicsObject):
         GraphicsObject.setExportMode(self, *args, **kwds)
         self.invalidate()
 
-    def mapPointsToDevice(self, pts):
-        warnings.warn(
-            "This is an internal method that is no longer being used. "
-            "Will be removed in 0.13",
-            DeprecationWarning, stacklevel=2
-        )
-        # Map point locations to device
-        tr = self.deviceTransform()
-        if tr is None:
-            return None
-
-        pts = fn.transformCoordinates(tr, pts)
-        pts -= self.data['sourceRect']['w'] / 2
-        pts = np.clip(pts, -2**30, 2**30) ## prevent Qt segmentation fault.
-
-        return pts
-
-    def getViewMask(self, pts):
-        warnings.warn(
-            "This is an internal method that is no longer being used. "
-            "Will be removed in 0.13",
-            DeprecationWarning, stacklevel=2
-        )
-        # Return bool mask indicating all points that are within viewbox
-        # pts is expressed in *device coordiantes*
-        vb = self.getViewBox()
-        if vb is None:
-            return None
-        viewBounds = vb.mapRectToDevice(vb.boundingRect())
-        w = self.data['sourceRect']['w'] / 2
-        mask = ((pts[0] + w > viewBounds.left()) &
-                (pts[0] - w < viewBounds.right()) &
-                (pts[1] + w > viewBounds.top()) &
-                (pts[1] - w < viewBounds.bottom())) ## remove out of view points
-
-        mask &= self.data['visible']
-        return mask
-
     @debug.warnOnException  ## raising an exception here causes crash
     def paint(self, p, *args):
         profiler = debug.Profiler()
@@ -1077,7 +987,7 @@ class ScatterPlotItem(GraphicsObject):
 
         if self.opts['pxMode'] is True:
             # Cull points that are outside view
-            viewMask = self._maskAt(self.getViewBox().viewRect())
+            viewMask = self._maskAt(self.viewRect())
 
             # Map points using painter's world transform so they are drawn with pixel-valued sizes
             pts = np.vstack([self.data['x'], self.data['y']])
@@ -1098,7 +1008,7 @@ class ScatterPlotItem(GraphicsObject):
                 frags[:, 6:10] = [1.0, 1.0, 0.0, 1.0]   # scaleX, scaleY, rotation, opacity
 
                 profiler('prep')
-                self._pixmapFragments.draw(p, len(frags), self.fragmentAtlas.pixmap)
+                self._pixmapFragments.draw(p, self.fragmentAtlas.pixmap)
                 profiler('draw')
             else:
                 # render each symbol individually
@@ -1217,12 +1127,17 @@ class ScatterPlotItem(GraphicsObject):
             # Show information about hovered points in a tool tip
             vb = self.getViewBox()
             if vb is not None and self.opts['tip'] is not None:
-                cutoff = 3
-                tip = [self.opts['tip'](x=pt.pos().x(), y=pt.pos().y(), data=pt.data())
-                       for pt in points[:cutoff]]
-                if len(points) > cutoff:
-                    tip.append('({} others...)'.format(len(points) - cutoff))
-                vb.setToolTip('\n\n'.join(tip))
+                if len(points) > 0:
+                    cutoff = 3
+                    tip = [self.opts['tip'](x=pt.pos().x(), y=pt.pos().y(), data=pt.data())
+                           for pt in points[:cutoff]]
+                    if len(points) > cutoff:
+                        tip.append('({} others...)'.format(len(points) - cutoff))
+                    vb.setToolTip('\n\n'.join(tip))
+                    self._toolTipCleared = False
+                elif not self._toolTipCleared:
+                    vb.setToolTip("")
+                    self._toolTipCleared = True
 
             self.sigHovered.emit(self, points, ev)
 
