@@ -53,17 +53,6 @@ def test_PlotWidget():
     for _ in range(5):
         assert_alldead(mkobjs())
 
-def test_GraphicsWindow():
-    def mkobjs():
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            w = pg.GraphicsWindow()
-        p1 = w.addPlot()
-        v1 = w.addViewBox()
-        return mkrefs(w, p1, v1)
-    
-    for _ in range(5):
-        assert_alldead(mkobjs())
 
 def test_ImageView():
     def mkobjs():
